/*
  Copyright (C) 2001 Kimmo Pekkola

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
/*
  $Header: //rainbox/cvsroot/Rainmeter/Plugin/MeasureNetOut.cpp,v 1.11 2002/04/26 18:24:16 rainy Exp $

  $Log: MeasureNetOut.cpp,v $
  Revision 1.11  2002/04/26 18:24:16  rainy
  Modified the Update method to support disabled measures.

  Revision 1.10  2002/04/01 15:37:25  rainy
  Moved part of the implementation to the base class.

  Revision 1.9  2002/03/31 09:58:54  rainy
  Added some comments

  Revision 1.8  2001/12/23 10:17:56  rainy
  The firstTime is now a member variable.

  Revision 1.7  2001/10/28 10:21:58  rainy
  Changes in the GetStat()

  Revision 1.6  2001/10/14 07:29:39  rainy
  Minor monifications to remove few warnings with VC.NET

  Revision 1.5  2001/09/26 16:27:14  rainy
  Changed the interfaces a bit.

  Revision 1.4  2001/09/01 13:00:10  rainy
  Slight changes in the interface. The value is now measured only once if possible.

  Revision 1.3  2001/08/19 09:14:55  rainy
  Added support for value invert.
  Also the CMeasure's ReadConfig is executed.

  Revision 1.2  2001/08/12 15:44:49  Rainy
  Adjusted Update()'s interface.
  Net throughput is now measured in bytes per second and not bits.

  Revision 1.1.1.1  2001/08/11 10:58:19  Rainy
  Added to CVS.

*/

#include "MeasureNetOut.h"

/*
** CMeasureNetOut
**
** The constructor
**
*/
CMeasureNetOut::CMeasureNetOut() : CMeasureNet()
{
	m_Counter = 0;
	m_FirstTime = true;
}

/*
** ~CMeasureNetOut
**
** The destructor
**
*/
CMeasureNetOut::~CMeasureNetOut()
{
}

/*
** Update
**
** Updates the current net out value. 
**
*/
bool CMeasureNetOut::Update(CMeterWindow& meterWindow)
{
	static UINT outOctets;
	int i;

	if (!CMeasureNet::Update(meterWindow)) return false;

	UpdateTable(m_Counter++);

	if(c_Table == NULL) return false;

	if(!m_FirstTime) 
	{
		m_Value = 0;
		for(i = 0; i < c_NumOfTables; i++)
		{
			// Ignore the loopback
			if(strcmp((char*)(c_Table->table[i].bDescr), "MS TCP Loopback interface") != 0)
			{
				m_Value += c_Table->table[i].dwOutOctets;
			}
		}
		UINT tmpValue = m_Value;
		m_Value -= outOctets;
		outOctets = tmpValue;
	}
	else
	{
		outOctets = 0;
		for(i = 0; i < c_NumOfTables; i++)
		{
			// Ignore the loopback
			if(strcmp((char*)c_Table->table[i].bDescr, "MS TCP Loopback interface") != 0)
			{
				outOctets += c_Table->table[i].dwOutOctets;
			}
		}

		m_FirstTime = false;
	}

	m_Total.QuadPart += m_Value;

	return true;
}

/*
** ReadConfig
**
** Reads the measure specific configs.
**
*/
void CMeasureNetOut::ReadConfig(const char* filename, const char* section)
{
	CMeasure::ReadConfig(filename, section);
	CMeasureNet::ReadConfig(filename, section, NET_OUT);
}

/*
** ReadStats
**
** Reads statistics from the registry.
**
*/
void CMeasureNetOut::ReadStats(HKEY hKey)
{
	CMeasureNet::ReadStats(hKey, NET_OUT);
}

/*
** WriteStats
**
** Writes statistics to the registry.
**
*/
void CMeasureNetOut::WriteStats(HKEY hKey)
{
	CMeasureNet::WriteStats(hKey, NET_OUT);
}

/*
** GetStats
**
** Returns the statistics as a string so that they can be shown in the about dialog.
**
*/
const char* CMeasureNetOut::GetStats()
{
	return CMeasureNet::GetStats(NET_OUT);
}
