/*
  Copyright (C) 2001 Kimmo Pekkola

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
/*
  $Header: //rainbox/cvsroot/Rainmeter/Plugin/Meter.h,v 1.8 2002/05/04 08:12:51 rainy Exp $

  $Log: Meter.h,v $
  Revision 1.8  2002/05/04 08:12:51  rainy
  Actions can be defined per meter.

  Revision 1.7  2002/04/26 18:22:03  rainy
  Added possibility to hide the meter.

  Revision 1.6  2002/03/31 09:58:54  rainy
  Added some comments

  Revision 1.5  2001/12/23 10:15:25  rainy
  Added ParseColor().

  Revision 1.4  2001/09/26 16:26:53  rainy
  Changed the interfaces a bit.

  Revision 1.3  2001/08/19 09:13:25  rainy
  Invert moved to the measures.

  Revision 1.2  2001/08/12 15:41:41  Rainy
  Adjusted Update()'s interface.
  Added invert measure.

  Revision 1.1.1.1  2001/08/11 10:58:19  Rainy
  Added to CVS.

*/

#ifndef __METER_H__
#define __METER_H__

#include <windows.h>
#include <list>

class CMeterWindow;
class CMeasure;

class CMeter
{
public:
	CMeter();
	virtual ~CMeter();

	virtual void ReadConfig(const char* filename, const char* section);
	virtual void Initialize(CMeterWindow& meterWindow);
	virtual void Draw(CMeterWindow& meterWindow) = 0;
	virtual void BindMeasure(std::list<CMeasure*>& measures);

	int GetH() { return m_H; };
	int GetW() { return m_W; };
	int GetX() { return m_X; };
	int GetY() { return m_Y; };

	std::string& GetRightMouseDownAction() { return m_RightMouseDownAction; };
	std::string& GetRightMouseUpAction() { return m_RightMouseUpAction; };
	std::string& GetLeftMouseDownAction() { return m_LeftMouseDownAction; };
	std::string& GetLeftMouseUpAction() { return m_LeftMouseUpAction; };

	void Hide() { m_Hidden = true; };
	void Show() { m_Hidden = false; };
	bool IsHidden() { return m_Hidden; };

	void SetName(const char* name) { m_Name = name; };
	const char* GetName() { return m_Name.c_str(); };

	static CMeter* Create(const char* meter);

protected:
	COLORREF ParseColor(const char* string);

	std::string m_Name;			// Name of the meter
	std::string m_MeasureName;	// Name of the measure this is bound to
	CMeasure* m_Measure;		// Pointer to the measure this meter is bound to
	int	m_X;					// X-position of the meter
	int	m_Y;					// Y-position of the meter
	int	m_W;					// Width of the meter
	int	m_H;					// Height of the meter
	bool m_Hidden;				// Status of the meter

	std::string m_RightMouseDownAction;	// Actions for left and right mouse buttons
	std::string m_RightMouseUpAction;
	std::string m_LeftMouseDownAction;
	std::string m_LeftMouseUpAction;
};

#endif
